import math
from .base import calc_base

def calc_entropy(password):
    """
    Calculates the entropy of a password in bits.
    Password entropy equation = log2(B^L)
        B -> Base
        L -> Length
    """
    base = calc_base(password)
    length = len(password)

    return round(math.log2(base**length), 2)


def validate(entropy, min_entropy=60.0):
    """
    Checks if the password's entropy meets the specified minimum requirement.

    Passwords should ideally be > 60.
    Great Passwords should be between 70-90.
    """
    return entropy >= min_entropy

#Estimates the time it would take to brute-force a password with the given entropy.
def estimate_bruteforce_time(entropy, guesses_per_second):
    """
    This code assumes that cracking time scales linearly with the number of possible combinations.

    While this might hold for straightforward brute-force attacks, more sophisticated attacks,
    such as dictionary attacks or those exploiting weaknesses in password hashing algorithms, may have different time complexities.
    """
    combinations = 2**entropy
    seconds_to_crack = combinations / guesses_per_second

    units = [("year", 60*60*24*365), ("day", 60*60*24), ("hour", 60*60), ("minute", 60), ("second", 1)]
    for unit, value in units:
        if seconds_to_crack >= value:
            amount = seconds_to_crack // value
            return f"{amount:.2f} {unit}{'s' if amount > 1 else ''}"
    return "Less than 1 second"



from setuptools import setup

with open('README.md') as file:
    long_description = file.read()

setup(
    name = 'enpass',
    version = '0.1.1',
    description = 'Simple entropy password validator.',
    long_description= long_description,
    long_description_content_type = 'text/markdown',
    author = 'Andres Ordonez',
    packages=['enpass'],
    url='https://github.com/Aresshu/enpass',
    license = 'MIT',
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python",
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords = "entropy password validator strength",
)


def calc_base(password):
    """
    Calculates the base of the entropy calculation based on the character types present in the password.
    
    Character Sets:
        26 lowercase letters
        26 uppercase letters
        10 digits
        36 special characters (assuming common ASCII set)

    Assuming a password is using at least 1 character of each set, the base would be: 98 (26 + 26 + 10 + 36)
    """
    
    base = 0    
    character_types = {
        'lower': False, 
        'upper': False, 
        'digit': False, 
        'special': False
        }

    for c in password:
        #If all characters have been encountered (ie: True) exit the loop early
        #This helps with efficiency for longer passwords
        if all(character_types.values()):
            break
        if c.islower() and not character_types['lower']:
            base += 26
            character_types['lower'] = True
        elif c.isupper() and not character_types['upper']:
            base += 26
            character_types['upper'] = True
        elif c.isdigit() and not character_types['digit']:
            base += 10
            character_types['digit'] = True
        elif not c.isalnum() and not character_types['special']:
            base += 36
            character_types['special'] = True

    return base